package com.perforce.maven.scm.provider.p4.command.list;

import java.util.List;

import org.apache.maven.scm.ScmFile;
import org.apache.maven.scm.ScmFileSet;
import org.apache.maven.scm.ScmTckTestCase;
import org.apache.maven.scm.ScmVersion;
import org.apache.maven.scm.command.list.ListScmResult;
import org.apache.maven.scm.provider.ScmProvider;
import org.codehaus.plexus.util.FileUtils;

import com.perforce.maven.scm.provider.p4.P4ScmTestUtils;

public class P4ListCommandTckTest
    extends ScmTckTestCase // this provider wont be able to handle List TCK, so we write out own
{

    /** {@inheritDoc} */
    public String getScmUrl()
        throws Exception
    {
        return P4ScmTestUtils.getScmUrl();
    }

    /** {@inheritDoc} */
    public void initRepo()
        throws Exception
    {
        P4ScmTestUtils.initRepo();
        resetToOriginalFiles();
    }

    private void resetToOriginalFiles()
        throws Exception
    {
        checkOut( getWorkingCopy(), getScmRepository() );
        ScmFileSet fileSet = new ScmFileSet( getWorkingCopy(), P4ScmTestUtils.getAddedIncludes(), null );
        getScmManager().getProviderByUrl( getScmUrl() ).remove( getScmRepository(), fileSet,
                                                                "Remove previous test files" );
        fileSet = new ScmFileSet( getWorkingDirectory(), P4ScmTestUtils.getTestIncludes(), null );
        getScmManager().getProviderByUrl( getScmUrl() ).edit( getScmRepository(), fileSet );
        P4ScmTestUtils.populateOriginalData( getBasedir(), getWorkingCopy() );
        checkIn( getWorkingCopy(), getScmRepository() );
        FileUtils.deleteDirectory( getWorkingDirectory() );
    }

    public void testListForSingleFile()
        throws Exception
    {
        ScmFileSet fileSet = new ScmFileSet( getWorkingCopy(), "**" );

        List<ScmFile> files = runList( fileSet, false );

        assertEquals( "The result of the list command doesn't have all the files in SCM: " + files, 4, files.size() );

    }

    private List<ScmFile> runList( ScmFileSet fileSet, boolean recursive )
        throws Exception
    {
        ScmProvider provider = getScmManager().getProviderByUrl( getScmUrl() );

        ListScmResult result = provider.list( getScmRepository(), fileSet, recursive, (ScmVersion) null );

        assertTrue( "SCM command failed: " + result.getCommandLine() + " : " + result.getProviderMessage()
                        + ( result.getCommandOutput() == null ? "" : ": " + result.getCommandOutput() ),
                    result.isSuccess() );

        return result.getFiles();
    }
}
